//----------------------------------------------------------------------------
// 
//	file: spikelayer3
// 
//	function: make spikelayer more robust to parameter change 
//		  implement global inhibition based on sum of map activity 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Fri Nov 27 04:49:20 CST 1998
// 
//	$Revision: 1.1 $ $Date: 2000/02/08 11:25:21 $
//	$Log: spikelayer3.h,v $
//	Revision 1.1  2000/02/08 11:25:21  yschoe
//	Initial revision
//
//	Revision 1.1  1999/02/09 07:42:25  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
#ifndef __SPIKE_LAYER3_H
#define __SPIKE_LAYER3_H

#ifdef SPIKE_QLEN
#undef SPIKE_QLEN
#undef SPIKE_QMASK
#define SPIKE_QLEN      8
#define SPIKE_QMASK 0xff
#endif

#include "spikelayer2.h"

//----------------------------------------------------------------------------
//	CLASS spikelayer
//	  - spiking lissom layer - derived from netlayer
//----------------------------------------------------------------------------
class spikelayer3 : public spikelayer2{

  // General idea:
  //	Put decayed spike activity in this->output[x][y], so that
  //	it could be generically used with other types of netlayers.
  //	No spikes actually goes through the connection this way.
  //	Thus the postsynaptic potential is calculated for all 
  //	targets receiving from a certain neuron.

public:

	spikelayer3(char *name, int cols, int rows, int exc_rad, int inh_rad,
                 int max_inp_layers, netlayer *feeding,
                 int inp_cols, int inp_rows, int  aff_rad, 
		 REAL synapse_decay, REAL inhloop_decay, 
		 REAL theta=0.0, REAL tau=0.0);
       ~spikelayer3();

	void activate(int calc_aff = 0);
	void calc_spike();
	void kill_recursive_lat();
	void print_stat(int cur_inp, int col, int row);

};

#endif
