//----------------------------------------------------------------------------
// 
//	file: util.c
// 
//	function: utility routines 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Thu Jul  9 00:36:30 CDT 1998
// 
//	$Revision: 1.5 $ $Date: 1998/11/17 12:47:19 $
//	$Log: util.cc,v $
//	Revision 1.5  1998/11/17 12:47:19  yschoe
//	added disp_binary() for displaying binary strings for a given decimal
//
//	Revision 1.4  1998/10/26 04:01:34  yschoe
//	*** empty log message ***
//
//	Revision 1.3  1998/10/16 10:08:04  yschoe
//	added warning(...) for warning only (do not exit).
//
//	Revision 1.2  1998/10/14 08:30:44  yschoe
//	changed error_exit() to handle variable argument
//
//	Revision 1.1  1998/07/23 01:07:05  yschoe
//	jul22 stable version (except netlayer)
//
//
//----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include "util.h"


//----------------------------------------------------------------------------
//	error_exit() 
//	  - exit after displaying error message
//----------------------------------------------------------------------------
// with vararg formatting
void error_exit(char *func, const char *format, ...) {

  FILE 	  *fp = stdout;
  va_list vlist;

  fprintf(fp,"%s(): ",func);
  va_start(vlist,format);
  vfprintf(fp,format,vlist);
  fprintf(fp," -- exiting");
  va_end(vlist);
  perror("");
  exit(0);

}

//----------------------------------------------------------------------------
//	warning() 
//	  - exit after displaying error message
//----------------------------------------------------------------------------
// with vararg formatting
void warning(char *func, const char *format, ...) {

  FILE 	  *fp = stdout;
  va_list vlist;

  fprintf(fp,"Warning in %s(): ",func);
  va_start(vlist,format);
  vfprintf(fp,format,vlist);
  fprintf(fp,"\n");
  va_end(vlist);

}


//----------------------------------------------------------------------------
//	disp_binary() 
//	  - display integer in binary format
//----------------------------------------------------------------------------
void disp_binary(int decimal) {

  int MASK = 0x1;
  int num = decimal;

  for (int i=0; i<sizeof(int)*4; ++i) {
	printf("%1d",MASK&num);
	num = num >> 1;
	
  }
}
