//----------------------------------------------------------------------------
// 
//	file: orpreftest.cc
// 
//	function: test or pref calc routine in netlayer class 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Sun Jan 10 00:13:35 CST 1999
//
//		orpreftest3 : rotation only -- centered at centroid
// 
//	$Revision: 1.1 $ $Date: 1999/02/09 07:56:44 $
//	$Log: orpreftest.cc,v $
//	Revision 1.1  1999/02/09 07:56:44  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------


#include <window.h>
#include <math.h>
#include <iostream.h>
#include <stream.h>
#include <nnview/real.h>
#include <nnview/matrix_window.h>
#include <nnview/plot_window.h>
#include <nnview/tool_window.h>
#include <nnview/util.h>
#include <nnview/geninput.h>
#include <nnview/disputil_window.h>

main_window     *mw;
matrix_window   *matw;
matrix_window   *matw2;
matrix_window   *matw3;
callback_button *runit;
callback_button *clearit;
geninput 	*input;
geninput 	*match;

REAL **mat;
REAL **ref;
REAL **mat3;

REAL gsigmax;
REAL gsigmay;
int locx, locy;

int N = 25;


//----------------------------------------------------------------------------
//	calc_single_or_pref() 
//	  - calculate or pref : copied from netlayer.cc
//----------------------------------------------------------------------------
REAL calc_single_or_pref(

	REAL **ref_pattern, 
	REAL **actual_pattern,
	int x_size, int y_size, 
	REAL sigmax, REAL sigmay) {

  // for 0 to 180 degrees, place gaussian blob on the COG of that
  // neuron's afferent weight matrix, and calculate the maximally
  // responding one, and return it in 0.0(0deg)-1.0(180deg) scale.
  int samples = 18;

  REAL mult = 180.0/(REAL)samples;
  REAL max_resp = -99999.9;
  int  max_idx  = 0;

  // Find approximate location of x and y Gaussian centers by
  // calculating max column and row sums
  REAL max_col_sum=0.0, max_row_sum=0.0, sum=0.0;
  int  max_col_idx, max_row_idx;
  for (int i=0; i<x_size; ++i) {
	
	sum = 0.0;
	for (int j=0; j<y_size; ++j) {
		sum += actual_pattern[i][j];
	}
	if (sum>max_col_sum) {
		max_col_sum = sum; max_col_idx = i;
	}
  }
	
  for (int j=0; j<y_size; ++j) {
	
	sum = 0.0;
	for (int i=0; i<x_size; ++i) {
		sum += actual_pattern[i][j];
	}
	if (sum>max_row_sum) {
		max_row_sum = sum; max_row_idx = j;
	}
  }

  // Scan
  for (int k=0; k<samples; ++k) { // for 0 - 180 degree rotation

      int diag = (int)(sqrt(x_size*x_size+y_size*y_size)*1.5);

      // generate gaussian
      for (int i=0; i<x_size; ++i)
          for (int j=0; j<y_size; ++j)
                ref_pattern[i][j] = 0.0;

      geninput::gaussian(ref_pattern, x_size, y_size,
			max_col_idx,max_row_idx,
			(REAL)x_size*10.0, (REAL)(N/4.0), (REAL)(k * mult));

      matw2->redraw();
      XFlush(display);

      // calc dotproduct (as a vector)
      REAL dot = 0.0;
      for (int i=0; i<x_size; ++i) {
          for (int j=0; j<y_size; ++j) {
              	dot += ref_pattern[i][j]*actual_pattern[i][j];
	  } 
      }
      //debug: cout << dot;

      if (dot>=max_resp) {
          max_resp = dot;
          max_idx  = k;
          //debug: cout << " " << k << " *" << endl;
	  match->clear_input();

	  match->add_input(max_col_idx,max_row_idx,
			(REAL)(x_size*2.0), (REAL)1.0, (REAL)(k * mult));
      	  matw3->redraw();
          XFlush(display);
      } else {
          //debug: cout << endl;
      }
  } // for : orientation

  return (max_idx/(REAL)(samples));

}


// the actual action takes place here.
void run() {

  //static int i, j;
  static int count;
  for (int i=0; i<36; ++i) {
  	input->clear_input();
  	match->clear_input();
 	REAL sigmax = gsigmax; //(REAL)(drand48()*2+1);
	REAL sigmay = gsigmay; //REAL sigmay = (REAL)(drand48()*8+1);
	REAL angle  = (count*10)%360; //REAL angle  = (REAL)(drand48()*360.0);
  	cout << "sigmax = " << sigmax 
	     << " sigmay = " << sigmay
	     << " angle = " << angle << endl;
  	input->add_input(
			// locx,locy,
			(int)((drand48()*N)/2.0+N/4.0),
			(int)((drand48()*N)/2.0+N/4.0),
			sigmax, sigmay, angle);
			// 10,10,
			// 3,(REAL)0.0,0.0);//(REAL)angle);	
			//(int)(drand48()*N),(int)(drand48()*N),
			//(REAL)(drand48()*8+3),(REAL)(drand48()*16+3),
			//(REAL)(drand48()*360.0));
  /*	input->add_input((int)(drand48()*N),(int)(drand48()*N),
			(REAL)(drand48()*8));
  */
  	matw->redraw();
	REAL ori=calc_single_or_pref(ref,input->input,N,N,sigmax,sigmay)*180.0 ;
	cout << "orientation = " << ori << endl;
  	XFlush(display);
  	count ++;
  }
  //cout << "run\n";
}

// clear mat
void clear() {
  disputil_window *dw = new disputil_window("disputil",mat,N,N,20);
  // input->clear_input();
  matw->redraw();
  XFlush(display);
  cout << "clean\n";
}

// Main
int main(int argc, char *argv[]) {

  if (argc != 5) {
  	cout << "usage: orpreftest <sigmax> <sigmay> <locx> <locy>" << endl;
	cout << " 	sigmax : length of gaussian bar " << endl;
	cout << " 	sigmay : width of gaussian bar " << endl;
	cout << " 	locx : x location of gaussian bar " << endl;
	cout << " 	locy : y location of gaussian bar " << endl;
	exit(0);
  }
  mat = new REAL* [N];
  for (int i=0; i<N; ++i) mat[i] = new REAL [N];
  mat3 = new REAL* [N];
  for (int i=0; i<N; ++i) mat3[i] = new REAL [N];
  ref = new REAL* [N];
  for (int i=0; i<N; ++i) ref[i] = new REAL [N];

  gsigmax = (REAL)atoi(argv[1]);
  gsigmay = (REAL)atoi(argv[2]);
  locx = atoi(argv[3]);
  locy = atoi(argv[4]);

  // setup input class
  input = new geninput(mat,N,N,INP_SQR,INP_NORM_SQRT);
  match = new geninput(mat3,N,N,INP_SQR,INP_NORM_SQRT);
  input->clear_input();

  // setup the windows
  mw = new main_window(argv[0],722,260);

  matw = new matrix_window(*mw,240,240,0,0,0,60,COLOR,N,N,mat);
  matw2 = new matrix_window(*mw,240,240,241,0,0,60,COLOR,N,N,ref);
  matw3 = new matrix_window(*mw,240,240,482,0,0,60,COLOR,N,N,mat3);

  runit = new callback_button(*mw,"Run",run,
                mw->width/4,20,mw->width/2,mw->height-20);

  clearit = new callback_button(*mw,"Clean",clear,
                mw->width/4,20,mw->width*1/4,mw->height-20);

   
  // add quit button
  new quit_button(*mw,mw->width/4,20,mw->width*3/4,mw->height-20);

  // start main loop
  mw->main_loop();

}
  
