#!/usr/bin/perl

$usage = <<'EOF';
# runsimul.pl
# Run several simulations with a given slissom2-2 config file 
# with only changing one (or few) parameters.
# 
# usage: runsimul.pl [-f] <simul_stub_file>
#
#	-f 	: force overwrite of generated conf files
#
# 	input:		stub  configuration file <simul_stub_file> with
#			lines that should be inserted for each trial in 
#			the following format.
# 		format:
#			stub config file is like any regular slissom conf
#			file, but it contains these keywords:
#	
#			COMMAND: <command> ... CONF ... BASE
#
#				CONF and BASE are replaced by the
#				current trial's conf file and the basename
#
#			NUM_TRIALS: <number of trials>
#			TRIAL<n>:  <slissom command>
#
#			for each trial <n>,
#	output: 	runsimul will generate conf files from the
#			stub file and run simulations on those
#			conf files one by one.
EOF

require "getopts.pl";
&Getopts('f');

if ($#ARGV<0) {
	print $usage;
	exit(0);
}

$conf = $ARGV[0];
($basename) = split(/\.conf/,$conf);

#----------------------------------------------------------------------------
# 1. Parse stub file
#----------------------------------------------------------------------------
open(STUB,"$conf");

$i= 0 ;
while (<STUB>) {

  if (/NUM_TRIALS/) {
	chop;
	($junk,$num_trials) = split(/:/);
 	print STDERR "basename = $basename\n";	
	print STDERR "Number of Trials = $num_trials\n";
	for ($k=0; $k<=$num_trials; ++$k) {
		$trial[$k] = "";
	}
	$trial_num_flag = 1;
	next;
  }

  if (/TRIAL/) {
	chop;
	($trial_str,$command) = split(/:/);
 	$trial_str =~ s/TRIAL//;
	print STDERR "TRIAL = $trial_str : $command\n";
	$trial[$trial_str] .= $command."\n";
	next;
  }

  if (/COMMAND/) {
	chop;
	$shellcommand = $_;
	$shellcommand =~ s/COMMAND://;
	print STDERR "shell command = $shellcommand\n";
 	next;
  }

  $lines[$i] = $_;
  $i++;

}
close(STUB);

if ($trial_num_flag != 1) {
	print STDERR "NUM_TRIALS not specified. Aborting.\n";
	exit(0);
}


#----------------------------------------------------------------------------
# 2. Generate conf files
#----------------------------------------------------------------------------

for ($i=0; $i<$num_trials; ++$i) {

  $fname = "$basename.t$i.conf";
  if ( -f $fname && (! $opt_f) ) {
	print STDERR "File $fname exists. Aborting.\n";
	exit(0);
  }
  open (FP,"> $fname");
  print FP "# $conf : trial $i\n";
  for ($k=0; $k<=$#lines; ++$k) {
	print FP $lines[$k];
  }
  print FP "\n\n\n# Trial $i : these parameters are varied\n";
  print FP $trial[$i];
  close (FP);
}
	

#----------------------------------------------------------------------------
# 3. execute
#----------------------------------------------------------------------------

for ($i=0; $i<$num_trials; ++$i) {

   $trial_command = $shellcommand;
   $trialconf = "$basename.t$i.conf";
   $trialbase = "$basename.t$i";
   $trial_command =~ s/CONF/$trialconf/g;
   $trial_command =~ s/BASE/$trialbase/g;
   print "Running: $trial_command\n";
   system("$trial_command");
}
     
